@@ -25,7 +25,10 @@ _QUERY_OPTIONS = {
     "tailable_cursor": 2,
     "slave_okay": 4,
     "oplog_replay": 8,
-    "no_timeout": 16}
+    "no_timeout": 16,
+    "await_data": 32,
+    "exhaust": 64,
+    "partial": 128}
 
 
 # TODO might be cool to be able to do find().include("foo") or
@@ -38,8 +41,8 @@ class Cursor(object):
     def __init__(self, collection, spec=None, fields=None, skip=0, limit=0,
                  timeout=True, snapshot=False, tailable=False, sort=None,
                  max_scan=None, as_class=None, slave_okay=False,
-                 _must_use_master=False, _is_command=False,
-                 **kwargs):
+                 await_data=False, partial=False, manipulate=True,
+                 _must_use_master=False, _is_command=False, **kwargs):
         """Create a new cursor.
 
         Should not be called directly by application developers - see
@@ -64,6 +67,12 @@ class Cursor(object):
             raise TypeError("snapshot must be an instance of bool")
         if not isinstance(tailable, bool):
             raise TypeError("tailable must be an instance of bool")
+        if not isinstance(slave_okay, bool):
+            raise TypeError("slave_okay must be an instance of bool")
+        if not isinstance(await_data, bool):
+            raise TypeError("await_data must be an instance of bool")
+        if not isinstance(partial, bool):
+            raise TypeError("partial must be an instance of bool")
 
         if fields is not None:
             if not fields:
@@ -91,6 +100,8 @@ class Cursor(object):
 
         self.__timeout = timeout
         self.__tailable = tailable
+        self.__await_data = tailable and await_data
+        self.__partial = partial
         self.__snapshot = snapshot
         self.__ordering = sort and helpers._index_document(sort) or None
         self.__max_scan = max_scan
@@ -98,9 +109,11 @@ class Cursor(object):
         self.__hint = None
         self.__as_class = as_class
         self.__slave_okay = slave_okay
+        self.__manipulate = manipulate
         self.__tz_aware = collection.database.connection.tz_aware
         self.__must_use_master = _must_use_master
         self.__is_command = _is_command
+        self.__query_flags = 0
 
         self.__data = []
         self.__connection_id = None
@@ -159,8 +172,12 @@ class Cursor(object):
         copy.__max_scan = self.__max_scan
         copy.__as_class = self.__as_class
         copy.__slave_okay = self.__slave_okay
+        copy.__await_data = self.__await_data
+        copy.__partial = self.__partial
+        copy.__manipulate = self.__manipulate
         copy.__must_use_master = self.__must_use_master
         copy.__is_command = self.__is_command
+        copy.__query_flags = self.__query_flags
         copy.__kwargs = self.__kwargs
         return copy
 
@@ -196,14 +213,17 @@ class Cursor(object):
     def __query_options(self):
         """Get the query options string to use for this query.
         """
-        options = 0
+        options = self.__query_flags
         if self.__tailable:
             options |= _QUERY_OPTIONS["tailable_cursor"]
-        if (self.__collection.database.connection.slave_okay or
-            self.__slave_okay):
+        if self.__slave_okay:
             options |= _QUERY_OPTIONS["slave_okay"]
         if not self.__timeout:
             options |= _QUERY_OPTIONS["no_timeout"]
+        if self.__await_data:
+            options |= _QUERY_OPTIONS["await_data"]
+        if self.__partial:
+            options |= _QUERY_OPTIONS["partial"]
         return options
 
     def __check_okay_to_chain(self):
@@ -212,6 +232,32 @@ class Cursor(object):
         if self.__retrieved or self.__id is not None:
             raise InvalidOperation("cannot set options after executing query")
 
+    def add_option(self, mask):
+        """Set arbitary query flags using a bitmask.
+        
+        To set the tailable flag:
+        cursor.add_option(2)
+        """
+        if not isinstance(mask, int):
+            raise TypeError("mask must be an int")
+        self.__check_okay_to_chain()
+
+        self.__query_flags |= mask
+        return self
+
+    def remove_option(self, mask):
+        """Unset arbitrary query flags using a bitmask.
+        
+        To unset the tailable flag:
+        cursor.remove_option(2)
+        """
+        if not isinstance(mask, int):
+            raise TypeError("mask must be an int")
+        self.__check_okay_to_chain()
+
+        self.__query_flags &= ~mask
+        return self
+
     def limit(self, limit):
         """Limits the number of results to be returned by this cursor.
 
@@ -607,7 +653,16 @@ class Cursor(object):
             raise StopIteration
         db = self.__collection.database
         if len(self.__data) or self._refresh():
-            next = db._fix_outgoing(self.__data.pop(0), self.__collection)
+            if self.__manipulate:
+                return db._fix_outgoing(self.__data.pop(0), self.__collection)
+            else:
+                return self.__data.pop(0)
         else:
             raise StopIteration
-        return next
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.__die()
+
