@@ -28,7 +28,7 @@ import struct
 import bson
 from bson.son import SON
 try:
-    from pymongo import _cbson
+    from pymongo import _cmessage
     _use_c = True
 except ImportError:
     _use_c = False
@@ -68,14 +68,11 @@ def insert(collection_name, docs, check_keys, safe, last_error_args):
     max_bson_size = 0
     data = __ZERO
     data += bson._make_c_string(collection_name)
-    bson_data = ""
-    for doc in docs:
-        encoded = bson.BSON.encode(doc, check_keys)
-        bson_data += encoded
-        max_bson_size = max(len(encoded), max_bson_size)
-    if not bson_data:
+    encoded = [bson.BSON.encode(doc, check_keys) for doc in docs]
+    if not encoded:
         raise InvalidOperation("cannot do an empty bulk insert")
-    data += bson_data
+    max_bson_size = max(map(len, encoded))
+    data += "".join(encoded)
     if safe:
         (_, insert_message) = __pack_message(2002, data)
         (request_id, error_message, _) = __last_error(last_error_args)
@@ -84,7 +81,7 @@ def insert(collection_name, docs, check_keys, safe, last_error_args):
         (request_id, insert_message) = __pack_message(2002, data)
         return (request_id, insert_message, max_bson_size)
 if _use_c:
-    insert = _cbson._insert_message
+    insert = _cmessage._insert_message
 
 
 def update(collection_name, upsert, multi, spec, doc, safe, last_error_args):
@@ -110,7 +107,7 @@ def update(collection_name, upsert, multi, spec, doc, safe, last_error_args):
         (request_id, update_message) = __pack_message(2001, data)
         return (request_id, update_message, len(encoded))
 if _use_c:
-    update = _cbson._update_message
+    update = _cmessage._update_message
 
 
 def query(options, collection_name,
@@ -131,7 +128,7 @@ def query(options, collection_name,
     (request_id, query_message) = __pack_message(2004, data)
     return (request_id, query_message, max_bson_size)
 if _use_c:
-    query = _cbson._query_message
+    query = _cmessage._query_message
 
 
 def get_more(collection_name, num_to_return, cursor_id):
@@ -143,7 +140,7 @@ def get_more(collection_name, num_to_return, cursor_id):
     data += struct.pack("<q", cursor_id)
     return __pack_message(2005, data)
 if _use_c:
-    get_more = _cbson._get_more_message
+    get_more = _cmessage._get_more_message
 
 
 def delete(collection_name, spec, safe, last_error_args):
