@@ -14,6 +14,7 @@
 
 """Tests for the dbref module."""
 
+import pickle
 import unittest
 import sys
 sys.path[0:0] = [""]
@@ -59,38 +60,55 @@ class TestDBRef(unittest.TestCase):
         self.assertRaises(AttributeError, bar)
 
     def test_repr(self):
-        self.assertEqual(repr(DBRef("coll", ObjectId("1234567890abcdef12345678"))),
+        self.assertEqual(repr(DBRef("coll",
+                                    ObjectId("1234567890abcdef12345678"))),
                          "DBRef('coll', ObjectId('1234567890abcdef12345678'))")
-        self.assertEqual(repr(DBRef(u"coll", ObjectId("1234567890abcdef12345678"))),
-                         "DBRef(u'coll', ObjectId('1234567890abcdef12345678'))")
+        self.assertEqual(repr(DBRef(u"coll",
+                              ObjectId("1234567890abcdef12345678"))),
+                         "DBRef(u'coll', ObjectId('1234567890abcdef12345678'))"
+                        )
         self.assertEqual(repr(DBRef("coll", 5, foo="bar")),
                          "DBRef('coll', 5, foo='bar')")
-        self.assertEqual(repr(DBRef("coll", ObjectId("1234567890abcdef12345678"), "foo")),
-                         "DBRef('coll', ObjectId('1234567890abcdef12345678'), 'foo')")
+        self.assertEqual(repr(DBRef("coll",
+                              ObjectId("1234567890abcdef12345678"), "foo")),
+                         "DBRef('coll', ObjectId('1234567890abcdef12345678'), "
+                         "'foo')")
         self.assertEqual(repr(DBRef("coll", 5, "baz", foo="bar", baz=4)),
                          "DBRef('coll', 5, 'baz', foo='bar', baz=4)")
 
     def test_cmp(self):
         self.assertEqual(DBRef("coll", ObjectId("1234567890abcdef12345678")),
                          DBRef(u"coll", ObjectId("1234567890abcdef12345678")))
-        self.assertNotEqual(DBRef("coll", ObjectId("1234567890abcdef12345678")),
-                            DBRef(u"coll", ObjectId("1234567890abcdef12345678"), "foo"))
-        self.assertNotEqual(DBRef("coll", ObjectId("1234567890abcdef12345678")),
+        self.assertNotEqual(DBRef("coll",
+                                  ObjectId("1234567890abcdef12345678")),
+                            DBRef(u"coll",
+                                  ObjectId("1234567890abcdef12345678"), "foo"))
+        self.assertNotEqual(DBRef("coll",
+                                  ObjectId("1234567890abcdef12345678")),
                             DBRef("col", ObjectId("1234567890abcdef12345678")))
-        self.assertNotEqual(DBRef("coll", ObjectId("1234567890abcdef12345678")),
+        self.assertNotEqual(DBRef("coll",
+                            ObjectId("1234567890abcdef12345678")),
                             DBRef("coll", ObjectId("123456789011")))
-        self.assertNotEqual(DBRef("coll", ObjectId("1234567890abcdef12345678")), 4)
-        self.assertEqual(DBRef("coll", ObjectId("1234567890abcdef12345678"), "foo"),
-                         DBRef(u"coll", ObjectId("1234567890abcdef12345678"), "foo"))
-        self.assertNotEqual(DBRef("coll", ObjectId("1234567890abcdef12345678"), "foo"),
-                            DBRef(u"coll", ObjectId("1234567890abcdef12345678"), "bar"))
+        self.assertNotEqual(DBRef("coll",
+                                  ObjectId("1234567890abcdef12345678")), 4)
+        self.assertEqual(DBRef("coll",
+                               ObjectId("1234567890abcdef12345678"), "foo"),
+                         DBRef(u"coll",
+                               ObjectId("1234567890abcdef12345678"), "foo"))
+        self.assertNotEqual(DBRef("coll",
+                                  ObjectId("1234567890abcdef12345678"), "foo"),
+                            DBRef(u"coll",
+                                  ObjectId("1234567890abcdef12345678"), "bar"))
 
     def test_kwargs(self):
-        self.assertEqual(DBRef("coll", 5, foo="bar"), DBRef("coll", 5, foo="bar"))
+        self.assertEqual(DBRef("coll", 5, foo="bar"),
+                         DBRef("coll", 5, foo="bar"))
         self.assertNotEqual(DBRef("coll", 5, foo="bar"), DBRef("coll", 5))
-        self.assertNotEqual(DBRef("coll", 5, foo="bar"), DBRef("coll", 5, foo="baz"))
+        self.assertNotEqual(DBRef("coll", 5, foo="bar"),
+                            DBRef("coll", 5, foo="baz"))
         self.assertEqual("bar", DBRef("coll", 5, foo="bar").foo)
-        self.assertRaises(KeyError, getattr, DBRef("coll", 5, foo="bar"), "bar")
+        self.assertRaises(AttributeError, getattr,
+                          DBRef("coll", 5, foo="bar"), "bar")
 
     def test_deepcopy(self):
         a = DBRef('coll', 'asdf', 'db', x=[1])
@@ -105,8 +123,22 @@ class TestDBRef(unittest.TestCase):
         self.assertEqual(a.x, [1])
         self.assertEqual(b.x, [2])
 
+    def test_pickling(self):
+        dbr = DBRef('coll', 5, foo='bar')
+        pkl = pickle.dumps(dbr)
+        dbr2 = pickle.loads(pkl)
+        self.assertEqual(dbr, dbr2)
 
+    def test_dbref_hash(self):
+        dbref_1a = DBRef('collection', 'id', 'database')
+        dbref_1b = DBRef('collection', 'id', 'database')
+        self.assertEquals(hash(dbref_1a), hash(dbref_1b))
 
+        dbref_2a = DBRef('collection', 'id', 'database', custom='custom')
+        dbref_2b = DBRef('collection', 'id', 'database', custom='custom')
+        self.assertEquals(hash(dbref_2a), hash(dbref_2b))
+
+        self.assertNotEqual(hash(dbref_1a), hash(dbref_2a))
 
 if __name__ == "__main__":
     unittest.main()
