@@ -15,6 +15,7 @@
 """Tests for the objectid module."""
 
 import datetime
+import pickle
 import warnings
 import unittest
 import sys
@@ -28,6 +29,7 @@ from bson.objectid import ObjectId
 from bson.tz_util import (FixedOffset,
                           utc)
 
+
 def oid(x):
     return ObjectId()
 
@@ -65,10 +67,14 @@ class TestObjectId(unittest.TestCase):
     def test_repr_str(self):
         self.assertEqual(repr(ObjectId("1234567890abcdef12345678")),
                          "ObjectId('1234567890abcdef12345678')")
-        self.assertEqual(str(ObjectId("1234567890abcdef12345678")), "1234567890abcdef12345678")
-        self.assertEqual(str(ObjectId("123456789012")), "313233343536373839303132")
-        self.assertEqual(ObjectId("1234567890abcdef12345678").binary, '\x124Vx\x90\xab\xcd\xef\x124Vx')
-        self.assertEqual(str(ObjectId('\x124Vx\x90\xab\xcd\xef\x124Vx')), "1234567890abcdef12345678")
+        self.assertEqual(str(ObjectId("1234567890abcdef12345678")),
+                         "1234567890abcdef12345678")
+        self.assertEqual(str(ObjectId("123456789012")),
+                         "313233343536373839303132")
+        self.assertEqual(ObjectId("1234567890abcdef12345678").binary,
+                         '\x124Vx\x90\xab\xcd\xef\x124Vx')
+        self.assertEqual(str(ObjectId('\x124Vx\x90\xab\xcd\xef\x124Vx')),
+                         "1234567890abcdef12345678")
 
     def test_cmp(self):
         a = ObjectId()
@@ -110,7 +116,7 @@ class TestObjectId(unittest.TestCase):
 
         self.assertEqual(utc, d2.tzinfo)
         d2 = d2.replace(tzinfo=None)
-        self.assert_(d2 - d1 < datetime.timedelta(seconds = 2))
+        self.assert_(d2 - d1 < datetime.timedelta(seconds=2))
 
     def test_from_datetime(self):
         d = datetime.datetime.utcnow()
@@ -119,10 +125,42 @@ class TestObjectId(unittest.TestCase):
         self.assertEqual(d, oid.generation_time.replace(tzinfo=None))
         self.assertEqual("0" * 16, str(oid)[8:])
 
-        aware = datetime.datetime(1993, 4, 4, 2, tzinfo=FixedOffset(555, "SomeZone"))
+        aware = datetime.datetime(1993, 4, 4, 2,
+                                  tzinfo=FixedOffset(555, "SomeZone"))
         as_utc = (aware - aware.utcoffset()).replace(tzinfo=utc)
         oid = ObjectId.from_datetime(aware)
         self.assertEqual(as_utc, oid.generation_time)
 
+    def test_pickling(self):
+        orig = ObjectId()
+        self.assertEqual(orig, pickle.loads(pickle.dumps(orig)))
+
+    def test_pickle_backwards_compatability(self):
+
+        # This string was generated by pickling an ObjectId in pymongo
+        # version 1.9
+        pickled_with_1_9 = (
+            "ccopy_reg\n_reconstructor\np0\n"
+            "(cbson.objectid\nObjectId\np1\nc__builtin__\n"
+            "object\np2\nNtp3\nRp4\n"
+            "(dp5\nS'_ObjectId__id'\np6\n"
+            "S'M\\x9afV\\x13v\\xc0\\x0b\\x88\\x00\\x00\\x00'\np7\nsb.")
+
+        # We also test against a hardcoded "New" pickle format so that we
+        # make sure we're backward compatible with the current version in
+        # the future as well.
+        pickled_with_1_10 = (
+            "ccopy_reg\n_reconstructor\np0\n"
+            "(cbson.objectid\nObjectId\np1\nc__builtin__\n"
+            "object\np2\nNtp3\nRp4\n"
+            "S'M\\x9afV\\x13v\\xc0\\x0b\\x88\\x00\\x00\\x00'\np5\nb."
+            )
+
+        oid_1_9 = pickle.loads(pickled_with_1_9)
+        oid_1_10 = pickle.loads(pickled_with_1_10)
+
+        self.assertEqual(oid_1_9, ObjectId("4d9a66561376c00b88000000"))
+        self.assertEqual(oid_1_9, oid_1_10)
+
 if __name__ == "__main__":
     unittest.main()
