@@ -25,6 +25,7 @@ sys.path[0:0] = [""]
 from nose.plugins.skip import SkipTest
 
 from pymongo.connection import Connection, _Pool
+from pymongo.errors import ConfigurationError
 from test_connection import get_connection
 
 N = 50
@@ -135,14 +136,17 @@ class TestPooling(unittest.TestCase):
         self.c[DB].unique.insert({"_id": "mike"})
         self.c[DB].unique.find_one()
 
+    def tearDown(self):
+        self.c = None
+
     def test_max_pool_size_validation(self):
         self.assertRaises(ValueError, Connection, max_pool_size=-1)
-        self.assertRaises(TypeError, Connection, max_pool_size='foo')
-        self.assertRaises(TypeError, Connection,
+        self.assertRaises(AssertionError, Connection, max_pool_size='foo')
+        self.assertRaises(ConfigurationError, Connection,
                           'mongodb://localhost/?maxPoolSize=-1')
-        self.assertRaises(TypeError, Connection,
+        self.assertRaises(ConfigurationError, Connection,
                           'mongodb://localhost/?maxPoolSize=foo')
-        self.assertRaises(TypeError, Connection,
+        self.assertRaises(ConfigurationError, Connection,
                           'mongodb://localhost/?maxPoolSize=5.5')
         c = Connection('mongodb://localhost/?maxPoolSize=5')
         self.assertEqual(c.max_pool_size, 5)
@@ -152,20 +156,26 @@ class TestPooling(unittest.TestCase):
     def test_no_disconnect(self):
         run_cases(self, [NoRequest, NonUnique, Unique, SaveAndFind])
 
+    def test_simple_disconnect(self):
+        self.c.test.stuff.find()
+        self.assertEqual(0, len(self.c._Connection__pool.sockets))
+        self.assertNotEqual(None, self.c._Connection__pool.sock)
+        self.c.end_request()
+        self.assertEqual(1, len(self.c._Connection__pool.sockets))
+        self.assertEqual(None, self.c._Connection__pool.sock)
+        self.c.disconnect()
+        self.assertEqual(0, len(self.c._Connection__pool.sockets))
+        self.assertEqual(None, self.c._Connection__pool.sock)
+
     def test_disconnect(self):
         run_cases(self, [SaveAndFind, Disconnect, Unique])
 
     def test_independent_pools(self):
-        p = _Pool(None, 10)
+        p = _Pool(10, 0)
         self.assertEqual([], p.sockets)
         self.c.end_request()
         self.assertEqual([], p.sockets)
 
-        # Sensical values aren't really important here
-        p1 = _Pool(5, 10)
-        self.assertEqual(None, p.socket_factory)
-        self.assertEqual(5, p1.socket_factory)
-
     def test_dependent_pools(self):
         c = get_connection()
         self.assertEqual(1, len(c._Connection__pool.sockets))
@@ -206,8 +216,10 @@ class TestPooling(unittest.TestCase):
         b_sock = b._Connection__pool.sockets[0]
         b.test.test.find_one()
         a.test.test.find_one()
-        self.assertEqual(b_sock, b._Connection__pool.socket())
-        self.assertEqual(a_sock, a._Connection__pool.socket())
+        self.assertEqual(b_sock,
+                         b._Connection__pool.get_socket(b.host, b.port)[0])
+        self.assertEqual(a_sock,
+                         a._Connection__pool.get_socket(a.host, a.port)[0])
 
     def test_pool_with_fork(self):
         if sys.platform == "win32":
@@ -259,7 +271,8 @@ class TestPooling(unittest.TestCase):
         self.assert_(a_sock.getsockname() != b_sock)
         self.assert_(a_sock.getsockname() != c_sock)
         self.assert_(b_sock != c_sock)
-        self.assertEqual(a_sock, a._Connection__pool.socket())
+        self.assertEqual(a_sock,
+                         a._Connection__pool.get_socket(a.host, a.port)[0])
 
     def test_max_pool_size(self):
         c = get_connection(max_pool_size=4)
