@@ -45,33 +45,40 @@ large = {"base_url": "http://www.example.com/test-me",
                             "no_of_js_attached": 10,
                             "no_of_images": 6
                             },
-         "harvested_words": ["10gen","web","open","source","application","paas",
-                             "platform-as-a-service","technology","helps",
-                             "developers","focus","building","mongodb","mongo"] * 20
+         "harvested_words": ["10gen", "web", "open", "source", "application",
+                             "paas", "platform-as-a-service", "technology",
+                             "helps", "developers", "focus", "building",
+                             "mongodb", "mongo"] * 20
          }
 
+
 def setup_insert(db, collection, object):
     db.drop_collection(collection)
 
+
 def insert(db, collection, object):
     for i in range(per_trial):
         to_insert = object.copy()
         to_insert["x"] = i
         db[collection].insert(to_insert)
 
+
 def insert_batch(db, collection, object):
     for i in range(per_trial / batch_size):
         db[collection].insert([object] * batch_size)
 
+
 def find_one(db, collection, x):
     for _ in range(per_trial):
         db[collection].find_one({"x": x})
 
+
 def find(db, collection, x):
     for _ in range(per_trial):
         for _ in db[collection].find({"x": x}):
             pass
 
+
 def timed(name, function, args=[], setup=None):
     times = []
     for _ in range(trials):
@@ -84,15 +91,19 @@ def timed(name, function, args=[], setup=None):
     print "%s%d" % (name + (60 - len(name)) * ".", per_trial / best_time)
     return best_time
 
+
 def main():
-    connection._TIMEOUT=60 # jack up the timeout
+    connection._TIMEOUT = 60  # jack up the timeout
     c = connection.Connection()
     c.drop_database("benchmark")
     db = c.benchmark
 
-    timed("insert (small, no index)", insert, [db, 'small_none', small], setup_insert)
-    timed("insert (medium, no index)", insert, [db, 'medium_none', medium], setup_insert)
-    timed("insert (large, no index)", insert, [db, 'large_none', large], setup_insert)
+    timed("insert (small, no index)", insert,
+          [db, 'small_none', small], setup_insert)
+    timed("insert (medium, no index)", insert,
+          [db, 'medium_none', medium], setup_insert)
+    timed("insert (large, no index)", insert,
+          [db, 'large_none', large], setup_insert)
 
     db.small_index.create_index("x", ASCENDING)
     timed("insert (small, indexed)", insert, [db, 'small_index', small])
@@ -101,17 +112,26 @@ def main():
     db.large_index.create_index("x", ASCENDING)
     timed("insert (large, indexed)", insert, [db, 'large_index', large])
 
-    timed("batch insert (small, no index)", insert_batch, [db, 'small_bulk', small], setup_insert)
-    timed("batch insert (medium, no index)", insert_batch, [db, 'medium_bulk', medium], setup_insert)
-    timed("batch insert (large, no index)", insert_batch, [db, 'large_bulk', large], setup_insert)
+    timed("batch insert (small, no index)", insert_batch,
+          [db, 'small_bulk', small], setup_insert)
+    timed("batch insert (medium, no index)", insert_batch,
+          [db, 'medium_bulk', medium], setup_insert)
+    timed("batch insert (large, no index)", insert_batch,
+          [db, 'large_bulk', large], setup_insert)
 
-    timed("find_one (small, no index)", find_one, [db, 'small_none', per_trial / 2])
-    timed("find_one (medium, no index)", find_one, [db, 'medium_none', per_trial / 2])
-    timed("find_one (large, no index)", find_one, [db, 'large_none', per_trial / 2])
+    timed("find_one (small, no index)", find_one,
+          [db, 'small_none', per_trial / 2])
+    timed("find_one (medium, no index)", find_one,
+          [db, 'medium_none', per_trial / 2])
+    timed("find_one (large, no index)", find_one,
+          [db, 'large_none', per_trial / 2])
 
-    timed("find_one (small, indexed)", find_one, [db, 'small_index', per_trial / 2])
-    timed("find_one (medium, indexed)", find_one, [db, 'medium_index', per_trial / 2])
-    timed("find_one (large, indexed)", find_one, [db, 'large_index', per_trial / 2])
+    timed("find_one (small, indexed)", find_one,
+          [db, 'small_index', per_trial / 2])
+    timed("find_one (medium, indexed)", find_one,
+          [db, 'medium_index', per_trial / 2])
+    timed("find_one (large, indexed)", find_one,
+          [db, 'large_index', per_trial / 2])
 
     timed("find (small, no index)", find, [db, 'small_none', per_trial / 2])
     timed("find (medium, no index)", find, [db, 'medium_none', per_trial / 2])
@@ -122,18 +142,24 @@ def main():
     timed("find (large, indexed)", find, [db, 'large_index', per_trial / 2])
 
 #     timed("find range (small, no index)", find,
-#           [db, 'small_none', {"$gt": per_trial / 4, "$lt": 3 * per_trial / 4}])
+#           [db, 'small_none',
+#            {"$gt": per_trial / 4, "$lt": 3 * per_trial / 4}])
 #     timed("find range (medium, no index)", find,
-#           [db, 'medium_none', {"$gt": per_trial / 4, "$lt": 3 * per_trial / 4}])
+#           [db, 'medium_none',
+#            {"$gt": per_trial / 4, "$lt": 3 * per_trial / 4}])
 #     timed("find range (large, no index)", find,
-#           [db, 'large_none', {"$gt": per_trial / 4, "$lt": 3 * per_trial / 4}])
+#           [db, 'large_none',
+#            {"$gt": per_trial / 4, "$lt": 3 * per_trial / 4}])
 
     timed("find range (small, indexed)", find,
-          [db, 'small_index', {"$gt": per_trial / 2, "$lt": per_trial / 2 + batch_size}])
+          [db, 'small_index',
+           {"$gt": per_trial / 2, "$lt": per_trial / 2 + batch_size}])
     timed("find range (medium, indexed)", find,
-          [db, 'medium_index', {"$gt": per_trial / 2, "$lt": per_trial / 2 + batch_size}])
+          [db, 'medium_index',
+           {"$gt": per_trial / 2, "$lt": per_trial / 2 + batch_size}])
     timed("find range (large, indexed)", find,
-          [db, 'large_index', {"$gt": per_trial / 2, "$lt": per_trial / 2 + batch_size}])
+          [db, 'large_index',
+           {"$gt": per_trial / 2, "$lt": per_trial / 2 + batch_size}])
 
 if __name__ == "__main__":
 #    cProfile.run("main()")
